#!/usr/bin/perl -w
#
#         Name : psummary2qdinfo.pl
#
#      Version : 0.10
#
#      Changed : 2005-12-01
# 
#       Author : Bas Couwenberg <sebastic@xs4all.nl>
#                original psummaryC.html parsing code 
#                by Dieter Vrancken <jackrabbit128@gmail.com>
# 
#      License : General Public License v2
#
#  Description : This script updates the point table and point history inside a
#                qdinfo.dat file with the data from a freshly downloaded 
#                psummaryC.html file.
#
#                It starts with reading the project credit data already in the 
#                qdinfo.dat, and compares that data with the more recent data 
#                in the psummaryC.html. If there are new projects which are not
#                in qdinfo.dat yet, they are added. And if there are projects
#                whos credit data has changed, this change is saved in the 
#                credit revision history of the project and the current credits
#                of that project are saved in the point table.
#
#        Usage : ./psummary2qdinfo.pl [OPTIONS]
# 		 Options:
# 		 -pu <url>  URL to the psummaryC.html file
# 		            default: http://fah-web.stanford.edu/psummaryC.html
# 		 -pf <file> Path to the psummaryC.html file
# 		            default: ./psummaryC.html
# 		 -q <file>  Path to the qdinfo.dat file
# 		            default: ./qdinfo.dat
# 		 -v         Verbose output
#
# Dependencies : * Requires some Perl Modules:
#                  - Date::Calc  
#                    http://search.cpan.org/~stbey/Date-Calc-5.4/
#                    Needed to get todays date, and convert the numerical month
#                    to a named month (example: 3 -> March)
#                  - LWP::UserAgent
#                    http://search.cpan.org/~gaas/libwww-perl-5.803/
#                    Needed to fetch the psummaryC.html from Stanfords webserver
#                * Needs to be run on a UNIX machine, 
#                  because the Operating System needs to use epoch 1970 time
#                * Requires read/write permissions to qdinfo.dat file and 
#                  psummaryC.html file. If the psummaryC.html does not exist,
#                  it needs write access to the directory in which it will be
#                  saved after it is downloaded.
#
#        Notes : Dick Howell originally ran a script like this one to generate
#                updated versions of qd, qd.exe and qdinfo.dat. The dates he
#                uses in qdinfo.dat are presumably in his timezone, which was 
#                in Waltham, MA, USA.
#                However I'm not exactly sure if this is correct. It could be
#                that those dates were in UTC or GMT.
#                The timestamps I generate are in the local timezone, the 
#                timezone used by the perl funcion localtime.
#                If someone can shed some light on this matter, I would love to
#                hear it!
#
#                Another time related issue are the epoch 2000 timestamps.
#                I assumed epoch 2000 referred to the J2000.0 epoch which is
#                used in astronomy and starts on January 1, 2000, 
#                11:58:55.816 UTC.
#                It turned out to be the epoch of Cosm, which starts on
#                January 1, 2000, 00:00:00 UTC.
#                qd.c uses an offset of 946684800 seconds between epoch 1970
#                and epoch 2000
#                
#                See the comments in decode_hextime() below for a little more 
#                info concerning epoch 2000.
#        
#    ChangeLog : v0.10:
#    		
#                Bas Couwenberg <sebastic@xs4all.nl>
#
#                * Fixed alignment of credit values. Project 2360 was worth 
#                  129400 credits, therefor the 5 characters used for the 
#                  credit were not enough
#    		
#                v0.9:
#
#                Bas Couwenberg <sebastic@xs4all.nl>
#
#		 * Fixed bug in psummary credit comparison. I saved the new 
#		   value in the history, should have saved the value up to now
#                
#                v0.8:
#
#                Bas Couwenberg <sebastic@xs4all.nl>
#
#		 * Fixed parsing of psummaryC.html with empty table cells
#		   On 2005-11-01 project 1312 had no "Number of Atoms"
#		   Thanks to Larry Plutchak for reporting this issue!
#
#                v0.7:
#    		 
#    		 Bas Couwenberg <sebastic@xs4all.nl>
#
#    		 * Update stanford server url, 
#    		   from vspx27.stanford.edu to fah-web.stanford.edu
#    		
#                v0.6:
#		 
#		 Bas Couwenberg <sebastic@xs4all.nl>
#
#		 * Fixed location of rph
#               
#                v0.5:
#		
#		 Bas Couwenberg <sebastic@xs4all.nl>
#
#		 * While reading the source of qd.c I've found the offset used
#		   in the epoch 2000 dates. The local epoch is expected to be
#		   epoch 1970, the epoch 2000 is the epoch used by Cosm.
#		   Epoch 2000 start date: January 1, 2000, 00:00:00 UTC
#    		
#                v0.4:
#                
#    		 Bas Couwenberg <sebastic@xs4all.nl>
#
#    		 * Updated report code, will only be shown if verbose output
#    		   is requested
#    		 * After the psummaryC.html data has been compared to the 
#    		   qdinfo.dat data, the progam exits if there were no changes
#
#                v0.3:
#    		 
#    		 Bas Couwenberg <sebastic@xs4all.nl>
#    		 
#    		 * Bugfix to report at end of the script, if there were no 
#    		   changes, perl complained about undeclared variables
#    		 * Bugfix to commandline parsing, won't fail anymore with 
#    		   "Unknown option" if commandline parameters are used
#    		 * Fixed several typos in the comments
#    		 * Added ChangeLog
#    		 * Added code to print the name and version of the script in 
#    		   the usage information, i.e when invoced with -h
#    		 * Added a little more info on the sections in qdinfo.dat
#    		 * Updated Dieters emailaddress
#    		 * Updated Description a bit to make it more clear what this
#    		   script does in general
#    		 * Updated the comment for the code where the qdinfo.dat data
#    		   is compared to the psummaryC.html data to check for changes
#    		   to make it more clear what is being done in general
#    		 * Removed unused variable $new_line which was used during
#    		   development of the point history recreating code. I tried
#    		   different methods to keep track of whether additional 
#    		   ph lines should be written, this variable was used but is
#    		   no longer needed
#
#    		 v0.2:
#    		 
#    		 Bas Couwenberg <sebastic@xs4all.nl>
#    		 
#    		 * Added a lot of documentation about this script, and about
#    		   the format of qdinfo.dat, so that we don't have to reinvent
#    		   the wheel
#    		 * Added point history handling, now updates those fields in
#    		   qdinfo.dat too
#    		 * Added psummaryC.html download code via LWP, no need to 
#    		   download it manually anymore
#
#    		v0.1:
#
#    		Bas Couwenberg <sebastic@xs4all.nl>
#
#    		* Initial release, as evolved from Dieter Vranckens psumdiff.pl
#

use strict;
use warnings;
use File::Basename;
use Date::Calc;
use LWP::UserAgent;

my $self          = 'psummary2qdinfo.pl';
my $version       = '0.9';
my $verbose       = 0;

# hash of counters
my %number_of = (  pt_projects      => 0,
                   ph_projects      => 0,
		   new_projects     => 0,
		   updated_projects => 0
                );

my $qdinfo_file   = './qdinfo.dat';
my $psummary_file = './psummaryC.html';
my $psummary_url  = 'http://fah-web.stanford.edu/psummaryC.html';

# if there are commandline parameters, parse them
# parameters are required in the form: -switch $value
if($ARGV[0]) {
	my $i = 0;
	foreach(@ARGV) {
		if($_ eq '-h' || $_ eq '--help') {
			print_usage($self, $version);
			exit;
		}
		elsif($_ eq '-v') {
			$verbose = 1;
		}
		elsif($_ eq '-pu') {
			if($ARGV[$i+1] && $ARGV[$i+1] ne '') {
				$psummary_url = $ARGV[$i+1];
			}
			else {
				print "Invalid parameter: $_\n\n";
				print_usage($self, $version);
				exit 1;
			}
		}
		elsif($_ eq '-pf') {
			if($ARGV[$i+1] && $ARGV[$i+1] ne '') {
				$psummary_file = $ARGV[$i+1];
			}
			else {
				print "Invalid parameter: $_\n\n";
				print_usage($self, $version);
				exit 1;
			}
		}
		elsif($_ eq '-q') {
			if($ARGV[$i+1] && $ARGV[$i+1] ne '') {
				$qdinfo_file  = $ARGV[$i+1];
			}
			else {
				print "Invalid parameter: $_\n\n";
				print_usage($self, $version);
				exit 1;
			}
		}
		$i++;
	}
}

# check the required permissions of the files
if(!check_permissions($psummary_file, $qdinfo_file, $verbose)) {
	print "\nErrors encountered.\n";
	print "Program aborted!\n";
	
	exit 1;
}

# create useragent object which will handle the HTTP request
my $ua  = LWP::UserAgent->new();
   $ua->agent("$self/$version");

print "Downloading URL: $psummary_url ... " if($verbose);

# download the psummaryC.html file 
my $req = HTTP::Request->new(GET => $psummary_url);
my $res = $ua->request($req);
if($res->is_success) {
	print "Success\n" if($verbose);
	
	print "Saving file to disc: $psummary_file\n" if($verbose);
	open(F, ">", $psummary_file) || die "Cannot open file: $psummary_file ($!)";
	print F $res->content;
	close F;

	print "\n" if($verbose);
}
else {
	print "Failed!\n" if($verbose);
	print "Error: Failed to retrieve URL! ($psummary_url)\n";
	print "Server returned: ".$res->code." ".$res->message."\n";

	print "\nProgram aborted!\n";	
	exit 1;
}

# get todays date and convert the numerical month to text
my ($year, $m, $day) = Date::Calc::Today();
my $month = Date::Calc::Month_to_Text($m);

# epoch 2000 time in hexidecimal
my $hextime = hextime();
   $hextime = sprintf("%08s", $hextime);

# define the components of the new qdinfo.dat
# the qdinfo.dat file has 3 sections:
# - the header        with the functional revision, the date it was generated 
#                     both as human readable date as well as epoch 2000 
#                     timestamp, and by whom the file was generated
# - the point table   with the current credits per project
# - the point history with the credit revisions per project

my $qdinfo_head   = "# <qdinfo.dat>\n\n";
   $qdinfo_head  .= "da $day $month $year (psummary2qdinfo)\n\n";
   $qdinfo_head  .= "# Latest functional revision number of \"qd\"\n\n";
   $qdinfo_head  .= "fr 032\n\n";
   $qdinfo_head  .= "# Point table generation time, epoch 2000\n\n";
   $qdinfo_head  .= "pg $hextime\n\n";
   $qdinfo_head  .= "# Point table entries (points per WU (project, 100 * points))\n\n";
my $point_table   = '';
my $point_history = "\n# Point history entries (project, 100 * old points, change date epoch 2000)\n\n";

# Define the hashes which will hold the project:credit lists
# 
# %credit_of is the hash which hold the credits per project
#
# %credit_of is a list in the form of:
# $credit_of{project_number} = credits
#
# %historic_credit_of is the hash which hold the credit revisions per project
#
# %historic_credit_of is a structure in the form of:
# $historic_credit_of{project_number} = @historic_credit
# 
# where @historic_credit is a list in the form of:
# $historic_credit[0] = credits:hexadecimal time
# $historic_credit[1] = credits:hexadecimal time

my %credit_of;
my %historic_credit_of;

# create a hash with project:credit pairs and 
# create a list of project changes from 
# the existing data in qdinfo.dat

open(F, $qdinfo_file) || die "Cannot open file: $qdinfo_file ($!)";
while(<F>) {
	next unless /^(pt|ph)/;	# only parse point table and point history
	
	# example point table lines:
	# pt 2051  16700  2052  16600  2053  13700  2054   6900  2055  12900
	# pt 2056  12800  2057  10600  2059   3800  2060   5700
	if(/^pt/) {
		s/^pt\s+//; # strip pt prefix
		
		# The point table lines contain dubble space separated pairs of
		# project and credit data. 
		# The first thing on each line are the pt characters to 
		# indicate that this line contains point table data.
		# The rest of the line is populated with the actual data, first
		# the project number followed by a space and then the amount of
		# credits for that project multiplied by 100
		#
		# To parse these lines, I remove the pt prefix, and create a
		# list of values by splitting the data on the line on one or
		# more spaces. To keep track of which value in the list is the
		# project number and which is the amount of credits, the 
		# variable $i flags the data. 
		
		my $i = 0;
		my $project = '';
		foreach(split /\s+/) {
			if($i == 0) {
				# the first value in the pair 
				# is the projectnumber
				
				# save the project number
				$project = $_;
				
				# increase the project counter
				$number_of{pt_projects} += 1;
				
				# next value in the list are the credits
				$i = 1;
			}
			else {
				# second value in the pair
				# are the credits of the project
	
				# add the credits to to the list
				# which we will use later to generate
				# the new qdinfo.dat file
				$credit_of{$project} = $_;

				# next value in the list is the project
				$i = 0;
			}
		}
	}
	# example point history lines:
	# ph 2018 15400 0A584D8C
	# ph 2019 20800 0A77D56C  2019 20100 0A72734C  2019 11600 0A71E6AC
	elsif(/^ph/) {
		# strip carriage return
		if(s/\r//) {}

		s/^ph\s+//; # strip ph prefix
		
		# The point history lines contain groups of credit revision 
		# history separted by double spaces. 
		# The first thing on each line are the ph characters to
		# indicate that this line contains point history data.
		# The rest of the line is populated with the actual data, first 
		# the project number followed by a space, then the amount of 
		# credits multiplied by 100, a space, and the epoch 2000 time
		# which indicated when the amount of credits was changed to 
		# that value.
		# Every line contains one or more credit revisions of only one
		# project. Because of the maximum of 80 character per line, 
		# there are at most 3 revision on one line.
		# If a project has more than 3 credit revision, additional 
		# ph lines are created to list all the revisions.
		# So it is very well posible to see a specific project which 
		# has several ph lines.
		# The revisions are ordered from latest up to the oldest. So 
		# the most recent revision is at the beginning of the line.
		#
		# To parse these lines, I remove the ph prefix, and create a
		# list of values by splitting the data on the line on one or 
		# more spaces. To keep track of which value in the list is the 
		# project numer, which is the amount of credits and which is 
		# the revision epoch 2000 time, the variable $i flags the data.
		
		my ($project, $credits, $hextime) = ('','','');
		my $i = 0;
		foreach(split /\s+/) {
			if($i == 0) {
				# the first value in the pair
				# is the project number
				
				# save the project number
				$project = $_;
				
				# increase the project counter
				$number_of{ph_projects} += 1;
			}
			elsif($i == 1) {
				# second value in the pair
				# is the credits*100 of the project
				
				# save the amount of credits
				$credits = $_;				
			}
			elsif($i == 2) {
				# third value in the pair
				# is the date of the change in epoch 2000
				# from this machines perspective
				
				# save the epoch 2000 time
				$hextime = $_;
				
				# add the revision data to the list 
				# for this project
				push @{$historic_credit_of{$project}}, "$credits:$hextime";

				# reset field counter/flag
				$i = -1; 
			}
			$i++;
		}
	}
}
close F;

# compare the project:credit pairs in qdinfo.dat with those in psummaryC.html
# changes are added to the lists which were populated with the already existing
# data in the qdinfo.dat file
print "Checking for updated projects\n" if($verbose);

open (F, "<", $psummary_file) || die "Cannot open file: $psummary_file ($!)";
while (<F>) {                           # read from file
	# Columns per tablerow:
	#
	# 0 Project Number
	# 1 Server IP
	# 2 Work Unit Name
	# 3 Number of Atoms
	# 4 Preferred (days)
	# 5 Final Deadline (days)
	# 6 Credit
	# 7 Frames
	# 8 Code
	# 9 Description
	#	
	# example line to parse:
	#
	#<TR bgcolor="#dfdfdf"> <TD>1311</TD><TD> 171.65.103.154 </TD>
	#<TD>p1311_1fnt_a10_91ps_CMnone</TD><TD>90068</TD><TD>   25.00</TD>
	#<TD>   37.00</TD><TD>  254.00</TD><TD>100</TD><TD>GROMACS</TD>
	#<TD><a href=http://fah-web.stanford.edu/cgi-bin/allprojects#1311>Description</a></TD>
	#<TD><font size=-1>sungjoo</font></TD></TR>
	
	next unless /cgi-bin/;          # only parse project lines
	chomp;                          # remove start/end junk
	
	s/<tr.*?>\s*<td>\s*//ig;        # strip <TR bgcolor="#dfdfdf"> <TD>
	s/\s*<\/td>\s*?<td>\s*/\;/ig;   # replace all </td><td> tags by ;
	s/\s*<\/td>\s*<\/tr>\s*//ig;    # strip </TD></TR>
	
	# example line;
	#243;171.64.122.127;p243_vil8MUreGS;6372;16.00;23.00;78.00;100;GROMACS
	#;<a href=http://fah-web.stanford.edu/cgi-bin/allprojects#243>Description</a>
	#;<font size=-1>rbrandman</font>
	
	my ($project, $credits) = (split(";", $_))[0,6]; # get data split by ;
	$credits  = $credits + 0;       # store\
	$credits *= 100;                # multiply by 100,
	                                # value is saved in qdinfo like that

	if($credit_of{$project}) {
		# the project exists in qdinfo.dat
		
		# update the credit value 
		# if it is different from the credits in qdinfo.dat
		if($credit_of{$project} != $credits) {
			print  "Updated project : ";
			printf("%4s", $project);
			print  "  new credits*100: ";
			printf("%6s", $credits);
			print  "  old credits*100: ";
			printf("%6s", $credit_of{$project});
			print "\n";
			
			# add the new credit revision at the beginning 
			# of the revision list of this project
			#unshift @{$historic_credit_of{$project}}, "$credits:$hextime";
			unshift @{$historic_credit_of{$project}}, "$credit_of{$project}:$hextime";
			
			# update the amount of credits for this project in the list
			$credit_of{$project} = $credits;
			
			# increase the counter of updated projects
			$number_of{updated_projects} += 1;
		}
	}
	else {
		# the project does not exist in qdinfo.dat yet
		
		print  "    New project : ";
		printf("%4s", $project);
		print  "  new credits*100: ";
		printf("%6s", $credits);
		print  "\n";
		
		# add this project to the list
		$credit_of{$project} = $credits;
		
		# increase the counter of new projects
		$number_of{new_projects} += 1;
	}
}
close F;

# no need to update the qdinfo.dat file
# if there are no new or updated projects
if($number_of{new_projects} == 0 && $number_of{updated_projects} == 0) {
	if($verbose) {
		print "\nNo data has changed, no need to update qdinfo.dat.\n";
	
		print "\n";
		print "Total number of projects in qdinfo.dat : ";
		print $number_of{pt_projects};
		print " (+ 0)";
		print "\n";
		print "Total number of point history entries  : ";
		print $number_of{ph_projects};
		print " (+ 0)";
		print "\n";
	}
	exit;
}

# recreate point table with updated project:value combinations
# each row starts with pt and should have 5 project:value combinations
my $i  = 0;
foreach my $project (sort { $a <=> $b } keys %credit_of) {
	# example pt line:
	# pt  101  7400   111   500   112   500   113   500   114   500
	
	if($i == 0) {
		$point_table .= "pt ";
	}
	if($i <= 4) {
		$point_table .= sprintf("%4s", $project);
		$point_table .= " ";
		$point_table .= sprintf("%6s", $credit_of{$project});
		$point_table .= "  " if($i < 4);
	}
	if($i == 4) {
		$point_table .= "\n";
		$i = -1;
	}
	$i++;
}
# last pt row possible does not contain 5 projects
if($point_table =~ s/  $/\n/) {}

# recreate point history table with updated credit revisions
# each row starts with ph and should have a max of 3 credit revisions per line
# if a project has more than 3 credit revisions addional ph lines need to be
# written to list them all

foreach my $project (sort { $a <=> $b } keys %historic_credit_of) {
	# example ph line:
	# ph 2019  20800 0A77D56C  2019  20100 0A72734C  2019  11600 0A71E6AC
	
	$point_history .= "ph ";
	my $i = 0;
	foreach my $project_history (@{$historic_credit_of{$project}}) {
		my ($credits, $hextime) = split /:/, $project_history;
		
		# this is not the first revision written on the line
		# add 2 spaces as a separator
		if($i > 0) {
			$point_history .= "  ";
		}

		$point_history .= sprintf("%4s", $project);
		$point_history .= " ";
		$point_history .= sprintf("%6s", $credits);
		$point_history .= " ";
		$point_history .= sprintf("%08s", $hextime);
		
		if($i == 2) {
			# wrote 3 revistion on one line
			# maximum reached
			
			$point_history .= "\n";
			
			if($#{$historic_credit_of{$project}} > 2) {
				# if this project has more than 3 revisions
				# write a new point history line
				
				$point_history .= "ph ";
			}
			
			$i = -1; # reset the field counter
		}

		$i++;
	}
	# add a newline if we didn't write a full line with 3 revisions
	$point_history .= "\n" unless($point_history =~ /\n$/);
}

# compose new qdinfo.dat file
my $qdinfo_data  = $qdinfo_head.$point_table.$point_history;

# save new qdinfo.dat
open(F, ">", $qdinfo_file) || die "Cannot open file: $qdinfo_file ($!)";
print F $qdinfo_data;
close F;

print "\nUpdated qdinfo.dat saved to: $qdinfo_file\n";

print "\n";
print "Total number of projects in qdinfo.dat : ";
print $number_of{pt_projects}+$number_of{new_projects};
print " (+ $number_of{new_projects})";
print "\n";
print "Total number of point history entries  : ";
print $number_of{ph_projects}+$number_of{updated_projects};
print " (+ $number_of{updated_projects})";
print "\n";

################################[ Subroutines ]################################

sub check_permissions {
	my ($psummary_file, $qdinfo_file, $verbose) = @_;

	print "Checking file permissions\n" if($verbose);

	# psummaryC.html
	# only check the file if it exist
	# check the parent directory if it has yet to be created
	#
	# needs read/write permissions to the file
	# or write permissions to the directory in which it is to be created
	
	print " local psummaryC.html : " if($verbose);
	if(-e $psummary_file) {
		if(!-w $psummary_file || !-r $psummary_file) {
			print "Error!\n" if($verbose);
			print "  file: $psummary_file\n";
			
			if(!-r $psummary_file) {
				print "   Cannot read file!\n";
			}
			if(!-w $psummary_file) {
				print "   Cannot write file!\n";
			}

			return undef;
		}
		else {
			print "OK\n" if($verbose);
		}
	}
	else {
		my $dir = dirname($psummary_file);
		if(!-w $dir) {
			print "Error!\n" if($verbose);
			print "  file: $psummary_file\n";
			print "   Cannot write directory!\n";
			
			return undef;
		}
		else {
			print "OK\n" if($verbose);
		}
	}

	# qdinfo.dat
	# needs read/write permissions
	print " local qdinfo.dat     : " if($verbose);
	if(!-r $qdinfo_file || !-w $qdinfo_file) {
		print "Error!\n" if($verbose);
		print "  file: $qdinfo_file\n";
		if(!-r $qdinfo_file) {
			print "   Cannot read file!\n";
		}
		if(!-w $qdinfo_file) {
			print "   Cannot write file!\n";
		}
		
		return undef;
	}
	else {
		print "OK\n" if($verbose);
	}
	print "\n" if($verbose);

	return 1;
}

sub decode_hextime {
	# convert a hexadecimal epoch 2000 value to human readable date string
	
	# Some notes on epoch 2000 vs epoch 1970
	# 
	# epoch 2000/Cosm
	# starts: January 1, 2000, 00:00:00 UTC
	#
	# epoch 1970/UNIX
	# starts: January 1, 1970, 00:00:00 UTC
	#
	# difference between the two dates: 946684800 seconds
	#
	# example epoch 2000 strings:
	# 0A8DA04A
	# 0817020C
	#
	# $date_string = scalar localtime (hex("0A8DA04A") + 946684800);
	#
	# add the seconds between epoch 2000 and epoch 1970 to the decimal 
	# value of the hex date to get the total amount of seconds since the
	# UNIX epoch convert that to human readable date through scalar 
	# localtime
	# 
	
	my $hextime = $_[0];
	
	# scalar localtime hex($_), uses the timezone of this machine
	# not Dick Howells who originally generated the qdinfo.dat files
	
	return scalar localtime (hex($hextime) + 946684800);
}

sub hextime {
	# convert current date to hex
	
	my $time  = time; 		  # seconds since epoch 1970
	   $time -= 946684800;  	  # seconds since epoch 2000 
	   $time  = sprintf("%X", $time); # seconds to hex in uppercase

	return $time;
}

sub print_usage {
	my ($self, $version) = @_;

	print "$self v$version\n\n";
	print "Usage : $self [OPTIONS]\n\n";
 	print "Options:\n";
 	print "-pu <url>  URL to the psummaryC.html file\n";
 	print "           default: http://fah-web.stanford.edu/psummaryC.html\n";
 	print "-pf <file> Path to local psummaryC.html file\n";
 	print "           default: ./psummaryC.html\n";
 	print "-q <file>  Path to the qdinfo.dat file\n";
 	print "           default: ./qdinfo.dat\n";
 	print "-v         Verbose output\n";
	print "\nSee the comments in the source code for more information\n";
}
